#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.interfaces import IAssociationContainerTarget
from pyams_content.component.association.zmi.interfaces import IAssociationsParentForm
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    PARAGRAPH_HIDDEN_FIELDS
from pyams_content.component.paragraph.interfaces.frame import IFrameParagraph, FRAME_PARAGRAPH_TYPE
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm
from pyams_skin.interfaces.tinymce import ITinyMCEConfiguration
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces import IPropertiesEditForm
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.association.zmi import AssociationsTable
from pyams_content.component.paragraph.frame import FrameParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm, get_json_paragraph_toolbar_refresh_event, \
    IParagraphEditFormButtons
from pyams_content.component.paragraph.zmi.container import ParagraphContainerTable, \
    ParagraphTitleToolbarViewletManager
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_form.form import ajax_config
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.event import get_json_switched_table_refresh_event, get_json_widget_refresh_event
from pyams_utils.adapter import adapter_config, ContextRequestAdapter
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from z3c.form import field, button
from zope.interface import implementer, Interface

from pyams_content import _


#
# Frame paragraph form interface and custom HTML edit widget
#

class IFrameParagraphForm(Interface):
    """Marker interface for framed text paragraph form"""


@adapter_config(context=(IFrameParagraphForm, IPyAMSLayer), provides=ITinyMCEConfiguration)
class FrameParagraphBodyEditorConfiguration(ContextRequestAdapter):
    """Custom configuration for 'body' widget editor"""

    configuration = {
        'ams-tinymce-menubar': False,
        'ams-tinymce-plugins': ['lists', ],
        'ams-tinymce-toolbar': 'undo redo | bold italic | bullist numlist',
        'ams-tinymce-toolbar1': False,
        'ams-tinymce-toolbar2': False,
        'ams-tinymce-height': 150
    }


#
# Framed text paragraph
#

@viewlet_config(name='add-frame-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=600)
class FrameParagraphAddMenu(BaseParagraphAddMenu):
    """Framed text paragraph add menu"""

    label = _("Framed text...")
    label_css_class = 'fa fa-fw fa-window-maximize'
    url = 'add-frame-paragraph.html'
    paragraph_type = FRAME_PARAGRAPH_TYPE


@pagelet_config(name='add-frame-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
@ajax_config(name='add-frame-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             base=BaseParagraphAJAXAddForm)
@implementer(IFrameParagraphForm)
class FrameParagraphAddForm(AdminDialogAddForm):
    """Framed text paragraph add form"""

    legend = _("Add new framed text paragraph")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-window-maximize'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IFrameParagraph).omit(*PARAGRAPH_HIDDEN_FIELDS)
    edit_permission = MANAGE_CONTENT_PERMISSION

    def create(self, data):
        return FrameParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@pagelet_config(name='properties.html', context=IFrameParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
@ajax_config(name='properties.json', context=IFrameParagraph, request_type=IPyAMSLayer,
             base=BaseParagraphAJAXEditForm)
@implementer(IFrameParagraphForm)
class FrameParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Framed text paragraph properties edit form"""

    prefix = 'frame_properties.'

    legend = _("Edit framed text paragraph properties")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-window-maximize'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IFrameParagraph).omit(*PARAGRAPH_HIDDEN_FIELDS)
    fields['renderer'].widgetFactory = RendererFieldWidget

    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(FrameParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            body_widget = self.widgets['body']
            for lang in body_widget.langs:
                widget = body_widget.widgets[lang]
                widget.id = '{id}_{name}'.format(id=widget.id, name=self.context.__name__)

    def get_ajax_output(self, changes):
        output = super(self.__class__, self).get_ajax_output(changes)
        if 'body' in changes.get(IFrameParagraph, ()):
            # refresh associations count markers
            parent = get_parent(self.context, IAssociationContainerTarget)
            output.setdefault('events', []).append(
                get_json_paragraph_toolbar_refresh_event(parent, self.request,
                                                         ParagraphContainerTable, ParagraphTitleToolbarViewletManager))
            # refresh associations table
            output.setdefault('events', []).append(
                get_json_switched_table_refresh_event(self.context, self.request, AssociationsTable))
        return output


@adapter_config(context=(IFrameParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@ajax_config(name='inner-properties.json', context=IFrameParagraph, layer=IPyAMSLayer,
             base=BaseParagraphAJAXEditForm)
@implementer(IInnerForm, IPropertiesEditForm, IAssociationsParentForm)
class FrameParagraphInnerEditForm(FrameParagraphPropertiesEditForm):
    """Framed text paragraph inner edit form"""

    legend = None

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IParagraphEditFormButtons)
        else:
            return button.Buttons()

    def get_ajax_output(self, changes):
        output = super(self.__class__, self).get_ajax_output(changes)
        updated = changes.get(IFrameParagraph, ())
        if 'renderer' in updated:
            output.setdefault('events', []).append(
                get_json_widget_refresh_event(self.context, self.request, FrameParagraphInnerEditForm, 'renderer'))
        return output
